"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emit = void 0;
const Keywords = require("../tokens/keywords");
const Delimiters = require("../tokens/delimiters");
const GenericTokens = require("../tokens/generic_tokens");
const TypeReferenceTokens = require("../tokens/type_reference_tokens");
const ModifierTokens = require("../tokens/modifier_tokens");
const AttributeEmitter = require("./attribute_emitter");
const MemberEmitter = require("./member_emitter");
const ScopeEmitter = require("./scope_emitter");
function emit(writer, _class) {
    AttributeEmitter.emitMany(writer, _class.attributes);
    emitDefinition(writer, _class);
    emitBody(writer, _class);
}
exports.emit = emit;
function emitDefinition(writer, _class) {
    const { access, isNew, isPartial, modifier, name, generics, baseType } = _class;
    const baseTypeTokens = baseType ? TypeReferenceTokens.emit(baseType) : [];
    const extensionTokens = baseTypeTokens.length ? [Delimiters.space, Delimiters.colon, Delimiters.space, ...baseTypeTokens] : [];
    writer
        .outputTabs()
        .write(...ModifierTokens.forAccess(access))
        .write(...ModifierTokens.forNew(isNew))
        .write(...ModifierTokens.forModifier(modifier))
        .write(...ModifierTokens.forPartial(isPartial))
        .write(Keywords._class, Delimiters.space, name)
        .write(...GenericTokens.emit(generics))
        .write(...extensionTokens)
        .writeLineNoTabs();
}
function emitBody(writer, _class) {
    ScopeEmitter.emit(writer, writer => MemberEmitter.emitMany(writer, _class.name, _class.members));
}
