"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emitOne = exports.emitMany = void 0;
const ArrayUtils = require("../utils/array_utils");
const Keywords = require("../tokens/keywords");
const Delimiters = require("../tokens/delimiters");
const NamespaceTokens = require("../tokens/namespace_tokens");
const NamespaceImportEmitter = require("./namespace_import_emitter");
const ScopeEmitter = require("./scope_emitter");
const TypeEmitter = require("./type_emitter");
function emitMany(writer, namespaces) {
    if (ArrayUtils.isFalsy(namespaces))
        return;
    namespaces.forEach((ns, i, arr) => emitOne(writer, ns, i === arr.length - 1));
}
exports.emitMany = emitMany;
function emitOne(writer, namespace, isLast) {
    emitDefinition(writer, namespace.path);
    emitBody(writer, namespace, isLast);
}
exports.emitOne = emitOne;
function emitDefinition(writer, path) {
    writer
        .outputTabs()
        .write(Keywords.namespace)
        .write(Delimiters.space)
        .write(...NamespaceTokens.emit(path))
        .writeLineNoTabs();
}
function emitBody(writer, namespace, isLast) {
    ScopeEmitter.emit(writer, writer => {
        NamespaceImportEmitter.emitMany(writer, namespace.imports);
        TypeEmitter.emitMany(writer, namespace.types);
        if (!isLast)
            writer.writeLine();
    });
}
